#!/usr/bin/python
##################################################################
# Copyright 2009 MIT Lincoln Laboratory  
# Project:            APP test
# Authors:            RWC
# Description:        Test harness for strawman.py
#
# Modifications:
# Date          Name           Modification
# ----          ----           ------------
# 07 Dec 2009   rwc            Initial release
# 08 Mar 2010   rwc            Fixed typo in usage statement.
#                              -d flag should be -t
# 08 Mar 2010   rwc            Fixed wrong variable in output
#                              message when script can't find
#                              client program to run
# 08 Mar 2010   rwc            Added command line argument to
#                              specify the name of the client
#                              program
##################################################################
import async_subprocess
import sys
import os
import getopt
import platform
import time
import datetime
import urllib2

def usage():
	print sys.argv[0] + ': -h -p <client program Directory> -n <client program Name> -t <test Directory> -f <exam filename>'
	print '   Where -h "help" will display this usage statement.'
	print '   -p <> specifies the directory where the program code is located'
	print '   -n <> specifies the name of the client program'
	print '   -t <> specifies the directory where the test sql file is located and'
	print '         where the results will be stored.'
	print '   -f <> specifies the filename containing the SQL queries'

# Read response from the PIPE.  Return a tuple with the result and reason.
def recv_answer(p, logfile):
	result = ''
	reason = ''
	started = 0
	c = ''
	while not c == ' ':
		while len(c) == 0:
			c = p.recv(1)
		if not started:
			logfile.write('  Receiving response: ' + str(datetime.datetime.now()) + '\n')
			started = 1
		else:
			started = 2
			result = result + c
			c = p.recv(1)

	while not c == '\r' or c == '\n':
		while len(c) == 0:
			c = p.recv(1)
		reason = reason + c
		c = p.recv(1)

	if started == 1:
		logfile.write('       Received null: ' + str(datetime.datetime.now()) + '\n')
	return (result, reason)

# Read from the PIPE to get the prompt indicating that the other side is reqdy.
def get_prompt(p):
	buffer = ''
	buffer = p.recv()
	while len(buffer) == 0:
		time.sleep(.3)
		buffer = p.recv()

	if buffer.find('>> ') == -1:
		print 'Protocol error: exiting'
		exit()
	else:
		return True

# Main body
progDir = 'src'
progName = 'strawman.py'
testDir = 'trials'
testFile = 'exam1.sql'

try:
	opts, args = getopt.getopt(sys.argv[1:], 'hp:n:t:f:', ['help', 'programDir=', 'programName=', 'testDir=', 'testFile='])
except getopt.GetoptError, err:
	print str(err)
	usage()
	sys.exit(2)

for option, argument in opts:
	if option == '-h':
		usage()
		sys.exit(0)
	elif option == '-p':
		progDir = argument.rstrip('/')
		progDir = progDir.rstrip('\\')
	elif option == '-n':
		progName = argument
	elif option == '-t':
		testDir = argument.rstrip('/')
		testDir = testDir.rstrip('\\')
	elif option == '-f':
		testFile = argument
	else:
		print 'Unhandled option.'
		usage()
		sys.exit(2)
if platform.system() == 'Windows':
	qFname = testDir + '/' + testFile
	qFname = qFname.replace('/','\\')
	oFname = testDir + '/' + testFile + '.log'
	oFname = oFname.replace('/','\\')
	if not os.path.exists(qFname):
		print 'Can not find test file: ' + qFname
		usage()
		sys.exit(2)
	queryFile = open(qFname, 'r')
	outputFile = open(oFname, 'w')
	if not os.path.exists(progDir + '\\strawman.py'):
		print 'Can not find client program: ' + progDir + '\\' + progName
		usage()
		sys.exit(2)
	execute, tail = ('python.exe ' + progDir + '\\' + progName, '\r\n')
else:
	if not os.path.exists(testDir + '/' + testFile):
		print 'Can not find test file: ' + testDir + '/' + testFile
		usage()
		sys.exit(2)
	queryFile = open(testDir + '/' + testFile, 'r')
	outputFile = open(testDir + '/' + testFile + '.log', 'w')
	if not os.path.exists(progDir + '/strawman.py'):
		print 'Can not find client program: ' + progDir + '/' + progName
		usage()
		sys.exit(2)
	execute, tail = (progDir + '/' + progName, '\n')
	
examLines = []
answer = []
reason = ''

line = ''
for line in queryFile:
	examLines.append(line)
queryFile.close()

testprog = async_subprocess.Popen(execute, stdin=async_subprocess.PIPE, stdout=async_subprocess.PIPE)
status = get_prompt(testprog)

i = 0
for line in examLines:
	recv_buffer = ''
	if i < 10:
		qNum = '0' + str(i)
	else:
		qNum = str(i)
	if line.strip().upper() == 'EXECUTE':
		outputFile.write('  Sending execute ' + qNum + ': ' + str(datetime.datetime.now()) + tail)
	elif not line.strip().upper().find('USER ') == -1:
		outputFile.write('    Changing user ' + qNum + ': ' + str(datetime.datetime.now()) + tail)
	else:
		outputFile.write('    Sending query ' + qNum + ': ' + str(datetime.datetime.now()) + tail)
	async_subprocess.send_all(testprog, line.strip() + tail)
	if line.strip().upper() == 'EXECUTE':
		answer = recv_answer(testprog, outputFile)
		outputFile.write('   Query complete ' + qNum + ': ' + str(datetime.datetime.now()) + tail)
		if answer[0] == 'ALLOWED':
			outputFile.write('   Result ' + qNum + ' ' + answer[0] + ': ' + str(datetime.datetime.now()) + tail)
		elif answer[0] == 'DENIED':
			outputFile.write('    Result ' + qNum + ' ' + answer[0] + ': ' + str(datetime.datetime.now()) + tail)
		else:
			outputFile.write('Bad Result ' + qNum + ' ' + answer[0] + ': ' + str(datetime.datetime.now()) + tail)
		reason = urllib2.urlopen(answer[1])
		if platform.system() == 'Windows':
			dFname = testDir + '/' + testFile + '-' + str(i) + '.dmp'
			dFname = dFname.replace('/','\\')
			dumpFile = open (dFname, 'w')
		else:
			dumpFile = open (testDir + '/' + testFile + '-' + str(i) + '.dmp', 'w')
		dumpFile.write(reason.read())
		dumpFile.close()
		outputFile.write('Encoding complete ' + qNum + ': ' + str(datetime.datetime.now()) + tail)
		outputFile.flush()
		i = i + 1
		status = get_prompt(testprog)
	elif not line.strip().upper().find('USER ') == -1:
		status = get_prompt(testprog)
		outputFile.write('User is now ' + line.strip().split()[1] + ' ' + qNum + ': ' + str(datetime.datetime.now()) + tail)
	else:
		status = get_prompt(testprog)
		outputFile.write('   Query received ' + qNum + ': ' + str(datetime.datetime.now()) + tail)
	
async_subprocess.send_all(testprog, 'DONE' + tail)
testprog.wait()
outputFile.close()
